@@ -1,4 +1,7 @@
 """Convert ASTs into L{pydoctor.model.Documentable} instances."""
+
+from __future__ import print_function
+
 from pydoctor import model, ast_pp
 
 from compiler import visitor, transformer, ast
@@ -324,7 +327,7 @@ class ModuleVistor(object):
                 defaults.append(ast_pp.pp(default))
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except Exception, e:
+            except Exception as e:
                 self.builder.warning("unparseable default",
                                      "%s: %s %r"%(e.__class__.__name__,
                                                   e, default))
