@@ -1,7 +1,15 @@
 """The command-line parsing and entry point."""
 
+from __future__ import print_function
+
 from pydoctor import model, zopeinterface
-from pydoctor.sphinx import SphinxInventory
+from pydoctor.sphinx import (
+    SphinxInventory,
+    prepareCache,
+    MAX_AGE_HELP,
+    USER_INTERSPHINX_CACHE
+)
+
 import sys, os, datetime
 
 BUILDTIME_FORMAT = '%Y-%m-%d %H:%M:%S'
@@ -9,7 +17,7 @@ BUILDTIME_FORMAT = '%Y-%m-%d %H:%M:%S'
 def error(msg, *args):
     if args:
         msg = msg%args
-    print >> sys.stderr, msg
+    print(msg, file=sys.stderr)
     sys.exit(1)
 
 def findClassFromDottedName(dottedname, optionname):
@@ -130,11 +138,6 @@ def getparser():
         '--buildtime', dest='buildtime',
         help=("Use the specified build time over the current time. "
               "Format: %s" % BUILDTIME_FORMAT))
-    parser.add_option(
-        '--livecheck', action='store_true', dest='livecheck',
-        default=False,
-        help=("Import and examine the modules too.  XXX not working "
-              "right now"))
     parser.add_option(
         '-v', '--verbose', action='count', dest='verbosity',
         default=0,
@@ -162,6 +165,34 @@ def getparser():
             "Use Sphinx objects inventory to generate links to external"
             "documetation. Can be repeated."))
 
+    parser.add_option(
+        '--enable-intersphinx-cache',
+        dest='enable_intersphinx_cache',
+        action='store_true',
+        default=False,
+        help="Enable Intersphinx cache."
+    )
+    parser.add_option(
+        '--intersphinx-cache-path',
+        dest='intersphinx_cache_path',
+        default=USER_INTERSPHINX_CACHE,
+        help="Where to cache intersphinx objects.inv files."
+    )
+    parser.add_option(
+        '--clear-intersphinx-cache',
+        dest='clear_intersphinx_cache',
+        action='store_true',
+        default=False,
+        help=("Clear the Intersphinx cache "
+              "specified by --intersphinx-cache-path."),
+    )
+    parser.add_option(
+        '--intersphinx-cache-max-age',
+        dest='intersphinx_cache_max_age',
+        default='1d',
+        help=MAX_AGE_HELP,
+    )
+
     return parser
 
 def readConfigFile(options):
@@ -197,7 +228,7 @@ def parse_args(args):
     return options, args
 
 def main(args):
-    import cPickle
+    from six.moves import cPickle
     options, args = parse_args(args)
 
     exitcode = 0
@@ -205,6 +236,11 @@ def main(args):
     if options.configfile:
         readConfigFile(options)
 
+    cache = prepareCache(clearCache=options.clear_intersphinx_cache,
+                         enableCache=options.enable_intersphinx_cache,
+                         cachePath=options.intersphinx_cache_path,
+                         maxAge=options.intersphinx_cache_max_age)
+
     try:
         # step 1: make/find the system
         if options.systemclass:
@@ -231,7 +267,7 @@ def main(args):
         # Once pickle support is removed, always instantiate System with
         # options and make fetchIntersphinxInventories private in __init__.
         system.options = options
-        system.fetchIntersphinxInventories()
+        system.fetchIntersphinxInventories(cache)
 
         system.urlprefix = ''
         if options.moresystems:
@@ -265,7 +301,7 @@ def main(args):
         try:
             system.buildtime = datetime.datetime.utcfromtimestamp(
                 int(os.environ['SOURCE_DATE_EPOCH']))
-        except ValueError, e:
+        except ValueError as e:
             error(e)
         except KeyError:
             pass
@@ -274,7 +310,7 @@ def main(args):
             try:
                 system.buildtime = datetime.datetime.strptime(
                     options.buildtime, BUILDTIME_FORMAT)
-            except ValueError, e:
+            except ValueError as e:
                 error(e)
 
         # step 2: add any packages and modules
@@ -308,9 +344,6 @@ def main(args):
 
         system.process()
 
-        if system.options.livecheck:
-            error("write this")
-
         if system.options.projectname is None:
             name = '/'.join([ro.name for ro in system.rootobjects])
             system.msg(
@@ -397,7 +430,7 @@ def main(args):
     except:
         if options.pdb:
             import pdb
-            pdb.post_mortem(sys.exc_traceback)
+            pdb.post_mortem(sys.exc_info()[2])
         raise
     return exitcode
 
