@@ -2,18 +2,21 @@
 Convert epydoc markup into renderable content.
 """
 
-import __builtin__
+from __future__ import print_function
+
+from six.moves import builtins
+from six.moves.urllib.parse import quote
 import exceptions
 import inspect
 import itertools
 import os
 import re
 import sys
-import urllib
 
 from twisted.web.template import Tag, tags, XMLString
 
 from pydoctor import model
+from pydoctor.epydoc.markup import DocstringLinker
 
 
 STDLIB_DIR = os.path.dirname(os.__file__)
@@ -21,14 +24,14 @@ STDLIB_URL = 'http://docs.python.org/library/'
 
 
 def link(o):
-    return o.system.urlprefix + urllib.quote(o.fullName()+'.html')
+    return o.system.urlprefix + quote(o.fullName()+'.html')
 
 
 def get_parser(formatname):
     try:
-        mod = __import__('epydoc.markup.' + formatname,
+        mod = __import__('pydoctor.epydoc.markup.' + formatname,
                          globals(), locals(), ['parse_docstring'])
-    except ImportError, e:
+    except ImportError as e:
         return None, e
     else:
         return mod.parse_docstring, None
@@ -59,8 +62,8 @@ def stdlib_doc_link_for_name(name):
                or sub_name in sys.builtin_module_names:
             return STDLIB_URL + sub_name + '.html#' + name
     part0 = parts[0]
-    if part0 in __builtin__.__dict__ and not part0.startswith('__'):
-        bltin = __builtin__.__dict__[part0]
+    if part0 in builtins.__dict__ and not part0.startswith('__'):
+        bltin = builtins.__dict__[part0]
         if part0 in exceptions.__dict__:
             return STDLIB_URL + 'exceptions.html#exceptions.' + name
         elif isinstance(bltin, type):
@@ -72,7 +75,7 @@ def stdlib_doc_link_for_name(name):
     return None
 
 
-class _EpydocLinker(object):
+class _EpydocLinker(DocstringLinker):
 
     def __init__(self, obj):
         self.obj = obj
@@ -88,7 +91,7 @@ class _EpydocLinker(object):
             p = obj.parent
             if isinstance(p, model.Module) and p.name == '__init__':
                 p = p.parent
-            linktext = link(p) + '#' + urllib.quote(obj.name)
+            linktext = link(p) + '#' + quote(obj.name)
         elif obj.documentation_location == model.DocLocation.OWN_PAGE:
             linktext = link(obj)
         else:
@@ -289,7 +292,7 @@ def html2stan(html):
 
 
 class Field(object):
-    """Like epydoc.markup.Field, but without the gross accessor
+    """Like pydoctor.epydoc.markup.Field, but without the gross accessor
     methods and with a formatted body."""
     def __init__(self, field, obj):
         self.tag = field.tag()
@@ -342,7 +345,6 @@ class FieldHandler(object):
     handle_rtype = handle_returntype
 
     def add_type_info(self, desc_list, field):
-        #print desc_list, field
         if desc_list and desc_list[-1].name == field.arg:
             if desc_list[-1].type is not None:
                 self.redef(field)
@@ -535,7 +537,7 @@ def doc2stan(obj, summary=False, docstring=None):
     doc = inspect.getdoc(crappit)
     try:
         pdoc = parse_docstring(doc, errs)
-    except Exception, e:
+    except Exception as e:
         errs = [e.__class__.__name__ +': ' + str(e)]
     if errs:
         reportErrors(source, errs)
@@ -544,7 +546,7 @@ def doc2stan(obj, summary=False, docstring=None):
     if pdoc is not None:
         try:
             crap = pdoc.to_html(_EpydocLinker(source))
-        except Exception, e:
+        except Exception as e:
             reportErrors(source, [e.__class__.__name__ +': ' + str(e)])
             return (boringDocstring(doc, summary),
                     [e.__class__.__name__ +': ' + str(e)])
