@@ -6,13 +6,15 @@ system being documented.  An instance of L{System} represents the whole system
 being documented -- a System is a bad of Documentables, in some sense.
 """
 
+from __future__ import print_function
+
 import datetime
 import imp
 import os
 import posixpath
 import sys
 import types
-import __builtin__
+from six.moves import builtins
 
 from pydoctor.sphinx import SphinxInventory
 
@@ -261,7 +263,7 @@ class Module(CanContainImportsDocumentable):
             return o.fullName()
         elif name in self._localNameToFullName_map:
             return self._localNameToFullName_map[name]
-        elif name in __builtin__.__dict__:
+        elif name in builtins.__dict__:
             return name
         else:
             return name
@@ -393,11 +395,11 @@ class System(object):
         else:
             i = '%s/%s'%(i,n)
         if self.verbosity(section) == 0 and sys.stdout.isatty():
-            print '\r'+i, msg,
+            print('\r'+i, msg, end='')
             sys.stdout.flush()
             if i == n:
                 self.needsnl = False
-                print
+                print()
             else:
                 self.needsnl = True
 
@@ -409,14 +411,14 @@ class System(object):
                 self.once_msgs.add((section, msg))
         if thresh <= self.verbosity(section) <= topthresh:
             if self.needsnl and wantsnl:
-                print
-            print msg,
+                print()
+            print(msg, end='')
             if nonl:
                 self.needsnl = True
                 sys.stdout.flush()
             else:
                 self.needsnl = False
-                print
+                print()
 
     def objForFullName(self, fullName):
         for system in [self] + self.moresystems:
@@ -430,7 +432,7 @@ class System(object):
         else:
             fn = '<None>'
         if self.options.verbosity > 0:
-            print fn, type, detail
+            print(fn, type, detail)
         self.warnings.setdefault(type, []).append((fn, detail))
 
     def objectsOfType(self, cls):
@@ -459,7 +461,7 @@ class System(object):
             for system in [self] + self.moresystems + self.subsystems:
                 if name in system.allobjects:
                     return system.allobjects[name]
-            raise KeyError, name
+            raise KeyError(name)
         self.__dict__.update(state)
         for system in [self] + self.moresystems + self.subsystems:
             if 'allobjects' not in system.__dict__:
@@ -603,7 +605,7 @@ class System(object):
         module = self.ensureModule(module_full_name)
         module.docstring = py_mod.__doc__
         self._introspectThing(py_mod, module, module)
-        print py_mod
+        print(py_mod)
 
     def addPackage(self, dirpath, parentPackage=None):
         if not os.path.exists(dirpath):
@@ -621,7 +623,7 @@ class System(object):
         package = self.ensurePackage(package_full_name)
         package.filepath = dirpath
         self.setSourceHref(package)
-        for fname in os.listdir(dirpath):
+        for fname in sorted(os.listdir(dirpath)):
             fullname = os.path.join(dirpath, fname)
             if os.path.isdir(fullname):
                 initname = os.path.join(fullname, '__init__.py')
@@ -726,13 +728,13 @@ class System(object):
 
     def process(self):
         while self.unprocessed_modules:
-            mod = iter(self.unprocessed_modules).next()
+            mod = next(iter(self.unprocessed_modules))
             self.processModule(mod)
 
 
-    def fetchIntersphinxInventories(self):
+    def fetchIntersphinxInventories(self, cache):
         """
         Download and parse intersphinx inventories based on configuration.
         """
         for url in self.options.intersphinx:
-            self.intersphinx.update(url)
+            self.intersphinx.update(cache, url)
