@@ -1,5 +1,7 @@
 """The classes that turn  L{Documentable} instances into objects we can render."""
 
+from __future__ import print_function
+
 from twisted.web.template import tags, Element, renderer, XMLFile
 
 from pydoctor import epydoc2stan, model
@@ -17,7 +19,7 @@ def getBetterThanArgspec(argspec):
     backargs.reverse()
     defaults = list(defaults)
     defaults.reverse()
-    kws = zip(backargs, defaults)
+    kws = list(zip(backargs, defaults))
     kws.reverse()
     return (args[:-len(kws)], kws)
 
@@ -60,7 +62,7 @@ class CommonPage(Element):
 
     @property
     def loader(self):
-        return XMLFile(util.templatefile('common.html'))
+        return XMLFile(util.templatefilepath('common.html'))
 
     def title(self):
         return self.ob.fullName()
@@ -276,7 +278,7 @@ class ClassPage(CommonPage):
 
     def mediumName(self, ob):
         r = [super(ClassPage, self).mediumName(ob)]
-        zipped = zip(self.ob.rawbases, self.ob.bases, self.ob.baseobjects)
+        zipped = list(zip(self.ob.rawbases, self.ob.bases, self.ob.baseobjects))
         if zipped:
             r.append('(')
             for i, (n, m, o) in enumerate(zipped):
@@ -300,7 +302,7 @@ class ClassPage(CommonPage):
             return []
         if baselists[0][0][0] == self.ob:
             del baselists[0]
-        return [item.fillSlots(
+        return [item.clone().fillSlots(
                           baseName=self.baseName(b),
                           baseTable=ChildTable(self.docgetter, self.ob,
                                                sorted(attrs, key=lambda o:-o.privacyClass)))
