@@ -1,3 +1,4 @@
+from __future__ import print_function
 
 from twisted.web.template import Element, renderer, XMLFile
 
@@ -5,7 +6,7 @@ from pydoctor.templatewriter import util
 
 class AttributeChild(Element):
 
-    loader = XMLFile(util.templatefile('attribute-child.html'))
+    loader = XMLFile(util.templatefilepath('attribute-child.html'))
 
     def __init__(self, docgetter, ob):
         self.docgetter = docgetter
