@@ -1,11 +1,17 @@
+from __future__ import print_function
+
+from twisted.python.compat import NativeStringIO
+
 from pydoctor import driver
-import sys, cStringIO
+
+import sys
+
 
 def geterrtext(*options):
     options = list(options)
     se = sys.stderr
-    f = cStringIO.StringIO()
-    print options
+    f = NativeStringIO()
+    print(options)
     sys.stderr = f
     try:
         try:
@@ -22,10 +28,6 @@ def test_invalid_option():
     err = geterrtext('--no-such-option')
     assert 'no such option' in err
 
-def test_no_do_nothing():
-    err = geterrtext()
-    assert "this invocation isn't going to do anything" in err
-
 def test_cannot_advance_blank_system():
     err = geterrtext('--make-html')
     assert 'forget an --add-package?' in err
@@ -51,3 +53,11 @@ def test_projectbasedir():
             "--project-base-dir", value])
     assert options.projectbasedirectory == value
 
+
+def test_cache_disabled_by_default():
+    """
+    Intersphinx object caching is disabled by default.
+    """
+    parser = driver.getparser()
+    (options, _) = parser.parse_args([])
+    assert not options.enable_intersphinx_cache
