@@ -1,9 +1,24 @@
 """Support for Zope interfaces."""
 
+from __future__ import print_function
+
 from pydoctor import model, ast_pp, astbuilder
 from compiler import ast
 import re
 
+
+class ZopeInterfaceModule(model.Module):
+    def setup(self):
+        super(ZopeInterfaceModule, self).setup()
+        self.implements_directly = [] # [name of interface]
+
+    @property
+    def allImplementedInterfaces(self):
+        """Return all the interfaces provided by this module
+        """
+        return list(self.implements_directly)
+
+
 class ZopeInterfaceClass(model.Class):
     isinterface = False
     isschemafield = False
@@ -34,34 +49,49 @@ class ZopeInterfaceClass(model.Class):
     @property
     def allImplementations(self):
         r = list(self.implementedby_directly)
-        stack = [self.system.objForFullName(n) for n in r]
+        stack = list(r)
         while stack:
             c = stack.pop(0)
             for sc in c.subclasses:
                 if sc.implementsOnly:
                     continue
                 stack.append(sc)
-                if sc.fullName() not in r:
-                    r.append(sc.fullName())
+                if sc not in r:
+                    r.append(sc)
         return r
 
-class Attribute(model.Documentable):
-    kind = "Attribute"
-    css_class = "attribute"
-    documentation_location = model.DocLocation.PARENT_PAGE
 
 class ZopeInterfaceFunction(model.Function):
     def docsources(self):
         for source in super(ZopeInterfaceFunction, self).docsources():
             yield source
-        if not isinstance(self.parent, model.Class):
+        if not isinstance(self.parent, (model.Class, model.Module)):
             return
         for interface in self.parent.allImplementedInterfaces:
             io = self.system.objForFullName(interface)
             if io is not None:
-                if self.name in io.contents:
-                    yield io.contents[self.name]
+                for io2 in io.allbases(include_self=True):
+                    if self.name in io2.contents:
+                        yield io2.contents[self.name]
 
+def addInterfaceInfoToModule(module, interfaceargs):
+    for arg in interfaceargs:
+        if not isinstance(arg, tuple):
+            fullName = module.expandName(ast_pp.pp(arg))
+        else:
+            fullName = arg[1]
+        module.implements_directly.append(fullName)
+        obj = module.system.objForFullName(fullName)
+        if obj is not None:
+            if not obj.isinterface:
+                obj.system.msg(
+                    'zopeinterface',
+                    'probable interface %r not marked as such'%obj,
+                    thresh=1)
+                obj.isinterface = True
+                obj.kind = "Interface"
+                obj.implementedby_directly = []
+            obj.implementedby_directly.append(module)
 
 def addInterfaceInfoToClass(cls, interfaceargs, implementsOnly):
     cls.implementsOnly = implementsOnly
@@ -69,9 +99,7 @@ def addInterfaceInfoToClass(cls, interfaceargs, implementsOnly):
         cls.implements_directly = []
     for arg in interfaceargs:
         if not isinstance(arg, tuple):
-            fullName = cls.dottedNameToFullName(ast_pp.pp(arg))
-            if fullName not in cls.system.allobjects:
-                fullName = cls.system.resolveAlias(fullName)
+            fullName = cls.expandName(ast_pp.pp(arg))
         else:
             fullName = arg[1]
         cls.implements_directly.append(fullName)
@@ -85,7 +113,7 @@ def addInterfaceInfoToClass(cls, interfaceargs, implementsOnly):
                 obj.isinterface = True
                 obj.kind = "Interface"
                 obj.implementedby_directly = []
-            obj.implementedby_directly.append(cls.fullName())
+            obj.implementedby_directly.append(cls)
 
 
 schema_prog = re.compile('zope\.schema\.([a-zA-Z_][a-zA-Z0-9_]*)')
@@ -115,20 +143,13 @@ class ZopeInterfaceModuleVisitor(astbuilder.ModuleVistor):
 
     def funcNameFromCall(self, node):
         str_base = ast_pp.pp(node.node)
-        return self.builder.current.dottedNameToFullName(str_base)
+        return self.builder.current.expandName(str_base)
 
     def visitAssign(self, node):
         # i would like pattern matching in python please
         # if match(Assign([AssName(?name, _)], CallFunc(?funcName, [Const(?docstring)])), node):
         #     ...
         sup = lambda : super(ZopeInterfaceModuleVisitor, self).visitAssign(node)
-        if isinstance(self.builder.current, model.Module) and \
-               ast_pp.pp(node) == 'Interface = interface.Interface\n':
-            # warner!!!
-
-            n2fn = self.builder.current._name2fullname
-            n2fn['Interface'] = 'zope.interface.Interface'
-            return sup()
         if len(node.nodes) != 1 or \
                not isinstance(node.nodes[0], ast.AssName) or \
                not isinstance(node.expr, ast.CallFunc):
@@ -152,13 +173,13 @@ class ZopeInterfaceModuleVisitor(astbuilder.ModuleVistor):
             return sup()
 
         def pushAttribute(docstring, kind):
-            attr = self.builder._push(Attribute, node.nodes[0].name, docstring)
+            attr = self.builder._push(model.Attribute, node.nodes[0].name, docstring)
             attr.linenumber = node.lineno
             attr.kind = kind
             if attr.parentMod.sourceHref:
                 attr.sourceHref = attr.parentMod.sourceHref + '#L' + \
                                   str(attr.linenumber)
-            self.builder._pop(Attribute)
+            self.builder._pop(model.Attribute)
 
         def extractStringLiteral(node):
             if isinstance(node, ast.Const) and isinstance(node.value, str):
@@ -172,7 +193,6 @@ class ZopeInterfaceModuleVisitor(astbuilder.ModuleVistor):
                 return node.args[0].value
 
         def handleSchemaField(kind):
-            #print node.expr
             descriptions = [arg for arg in node.expr.args if isinstance(arg, ast.Keyword)
                             and arg.name == 'description']
             docstring = None
@@ -205,6 +225,13 @@ class ZopeInterfaceModuleVisitor(astbuilder.ModuleVistor):
         if meth is not None:
             meth(base, node)
 
+    def visitCallFunc_zope_interface_moduleProvides(self, funcName, node):
+        if not isinstance(self.builder.current, model.Module):
+            self.default(node)
+            return
+
+        addInterfaceInfoToModule(self.builder.current, node.args)
+
     def visitCallFunc_zope_interface_implements(self, funcName, node):
         if not isinstance(self.builder.current, model.Class):
             self.default(node)
@@ -214,7 +241,7 @@ class ZopeInterfaceModuleVisitor(astbuilder.ModuleVistor):
     visitCallFunc_zope_interface_implementsOnly = visitCallFunc_zope_interface_implements
 
     def visitCallFunc_zope_interface_classImplements(self, funcName, node):
-        clsname = self.builder.current.dottedNameToFullName(
+        clsname = self.builder.current.expandName(
             ast_pp.pp(node.args[0]))
         if clsname not in self.system.allobjects:
             self.builder.system.msg(
@@ -231,8 +258,7 @@ class ZopeInterfaceModuleVisitor(astbuilder.ModuleVistor):
         cls = self.builder.current.contents[node.name]
         if 'zope.interface.interface.InterfaceClass' in cls.bases:
             cls.isinterfaceclass = True
-        if len([b for b in cls.bases
-                if namesInterface(self.system, b)]) > 0:
+        if any(namesInterface(self.system, b) for b in cls.bases):
             cls.isinterface = True
             cls.kind = "Interface"
             cls.implementedby_directly = []
@@ -249,7 +275,7 @@ class ZopeInterfaceASTBuilder(astbuilder.ASTBuilder):
 
 
 class ZopeInterfaceSystem(model.System):
+    Module = ZopeInterfaceModule
     Class = ZopeInterfaceClass
     Function = ZopeInterfaceFunction
     defaultBuilder = ZopeInterfaceASTBuilder
-
